from subprocess import run
from pathlib import Path

base_directory = Path("Videos\\encodes")
output_dir = base_directory / "output"
output_dir.mkdir(parents=True, exist_ok=True)

videos = [
    {  # https://4kmedia.org/samsung-wonderland-two-hdr-uhd-4k-demo/
        "name": "wonderland_two",
        "source": base_directory / "Samsung Wonderland Two HDR UHD 4K Demo.ts",
        "master_display": "G(13250,34500)B(7500,3000)R(34000,16000)WP(15635,16450)L(10000000,500)",
        "fps": "24000/1001",
        "resolution": "4k",
    },
    {   # https://developer.dolby.com/tools-media/sample-media/video-streams/dolby-vision-streams/
        "name": "glass_blowing",
        "source": base_directory / "P81_GlassBlowing2_3840x2160@59.94fps_15200kbps_fmp4.mp4",
        "master_display": "G(8500,39850)B(6550,2300)R(35400,14600)WP(15635,16450)L(40000000,50)",
        "cll": "0,0",
        "fps": "60000/1001",
        "resolution": "4k",
    },
]


def compare_vmaf(original, converted: Path, resolution, fps, **_):
    model = "vmaf_4k_v0.6.1.json" if resolution == "4k" else "vmaf_v0.6.1.json"
    log_path = f"{converted.name}.json"
    base_command = f'"FFmpeg\\bin\\ffmpeg.EXE" -y -i "{converted}" -i "{original}" -lavfi "[0:v]fps=fps={fps}[input0_0];[1:v]fps=fps={fps}[input1_0];[input0_0][input1_0]libvmaf=log_fmt=json:model_path={model}:log_path={log_path}:ssim=1:psnr=1:n_threads=4" -report -f null -'
    result = run(base_command, cwd=base_directory, shell=True)



def convert_nvenc(output, bitrate, source, master_display, cll=None, **_):
    cll = f'--max-cll "{cll}"' if cll else ""
    base_command = f'"C:\\apps\\NVEncC_5.37_x64\\NVEncC64.exe" -i "{source}" --video-metadata clear --chapter-copy -c hevc --vbr {bitrate} --bframes 3 --ref 3 --preset quality --tier high --lookahead 16 --aq --aq-strength 0 --colormatrix bt2020nc --transfer smpte2084 --colorprim bt2020 --master-display "{master_display}" {cll} --output-depth 10 --multipass 2pass-full --mv-precision Q-pel --chromaloc auto --colorrange auto --avsync cfr -o "{output}"'
    print(base_command)
    run(base_command, shell=True).check_returncode()


def convert_qsv(output, bitrate, source, master_display, cll=None, **_):
    cll = f'--max-cll "{cll}"' if cll else ""
    base_command = f'"C:\\apps\\QSVEncC_5.06_x64\\QSVEncC64.exe" -i "{source}" --video-metadata clear --chapter-copy -c hevc --vbr {bitrate} --bframes 3 --ref 3 --quality best --tier high --la-depth 16  --la-quality slow --extbrc --colormatrix bt2020nc --transfer smpte2084 --colorprim bt2020 --master-display "{master_display}" {cll} --output-depth 10  --chromaloc auto --colorrange auto --avsync vfr -o "{output}"'
    run(base_command, shell=True).check_returncode()


def convert_amf(output, bitrate, source, master_display, cll=None, **_):
    cll = f'--max-cll "{cll}"' if cll else ""
    #  --bframes 3 --ref 3  --tier high 
    base_command = f'"C:\\apps\\AMFEncC_x64\\AMFEncC64.exe" --avsw -i "{source}" --video-metadata clear --chapter-copy -c hevc --vbr {bitrate} --ref 3 --preset slow --pe --tier High --colormatrix bt2020nc --transfer smpte2084 --colorprim bt2020 --master-display "{master_display}" {cll} --output-depth 10  --chromaloc auto --colorrange auto --avsync cfr -o "{output}"'
    run(base_command, shell=True).check_returncode()


if __name__ == '__main__':
    for video in videos:
        for bitrate in ("2500", "5000", "7500", "10000", "12500"):
            print(f"Converting {video['name']} at {bitrate}k")
            
            nvenc_video = output_dir / f"{video['name']}_{bitrate}_nvenc.mp4"
            if not nvenc_video.exists():
                convert_nvenc(output=nvenc_video, bitrate=bitrate, **video)
                compare_vmaf(video["source"], nvenc_video, **video)
                
            qsv_video = output_dir / f"{video['name']}_{bitrate}_qsv.mp4"
            if not qsv_video.exists():
                convert_qsv(output=qsv_video, bitrate=bitrate, **video)
                compare_vmaf(video["source"], qsv_video, **video)

            amf_video = output_dir / f"{video['name']}_{bitrate}_amf.mp4"
            if not amf_video.exists():
                convert_amf(output=amf_video, bitrate=bitrate, **video)
                compare_vmaf(video["source"], amf_video, **video)
